<?php
/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/ZendSkeletonApplication for the canonical source repository
 * @copyright Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace Application;

use Zend\Db\ResultSet\ResultSet;
use Zend\Db\TableGateway\TableGateway;
use Zend\Mvc\ModuleRouteListener;
use Zend\Mvc\MvcEvent;
use Zend\Debug\Debug;
use Zend\Session\Config\SessionConfig;
use Zend\Session\SessionManager;
use Zend\Session\Container;
use Application\view\helper\ImagesPagination;
use Application\view\helper\UltimasPagination;
use Zend\EventManager\EventInterface;
use Zend\Db\TableGateway\Feature;
use Application\Model\DateHelper;
use Application\Plugin\Tools;
use Application\Model\CustomerTable;
use Application\Model\RequestTable;
use Application\Model\UsersTable;

class Module
{
    public function onBootstrap(MvcEvent $e)
    {
        $eventManager        = $e->getApplication()->getEventManager();
        $moduleRouteListener = new ModuleRouteListener();
        $moduleRouteListener->attach($eventManager);
		
		$application = $e->getParam('application');
        $config = $e->getApplication()->getServiceManager()->get('Config');
		$viewModel = $e->getApplication()->getMvcEvent()->getViewModel();
		
		$sessionConfig = new SessionConfig();
		$sessionConfig->setOptions($config['session']);
		$sessionManager = new SessionManager($sessionConfig);
		$sessionManager->start();
		
		Container::setDefaultManager($sessionManager);
		
		$userSessionAuth = new Container('userSessionAuth');
		$viewModel->setVariables(array(
            'layout' => $config["layout"],
            'ws' => $config["ws"],
            'loggedIn' => $userSessionAuth->offsetExists('loggedIn')
        ));
    }

    public function getConfig()
    {
        return include __DIR__ . '/config/module.config.php';
    }

    public function getAutoloaderConfig()
    {
        return array(
            'Zend\Loader\StandardAutoloader' => array(
                'namespaces' => array(
                    __NAMESPACE__ => __DIR__ . '/src/' . __NAMESPACE__,
                    'WSClientLib' => getenv("APPLICATION_PATH").'/vendor/WSClientLib',
                ),

            ),
        );
    }
	
	public function getViewHelperConfig()
    {
        return array(
            'factories' => array(
				'imagespagination' => function() {
					return new \Application\View\Helper\ImagesPagination();
                },
				'ultimaspagination' => function() {
					return new \Application\View\Helper\UltimasPagination();
                },
				'breadcrumbs' => function() {
					return new \Application\View\Helper\Breadcrumbs();
                },
            ),
        );
    }
	
	public function getServiceConfig()
	{
		return array (
			'factories' => array(

                'Application\Model\CustomerTable' =>  function($sm)
                {
                    $tableGateway = $sm->get('CustomerGateway');
                    $table = new CustomerTable($tableGateway);
                    return $table;
                },
                
                'CustomerGateway' => function($sm)
                {
                    $dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
                    $resultSetPrototype = new ResultSet();
                    
                    return new TableGateway('va_customer', $dbAdapter, new Feature\RowGatewayFeature('va_customer_id'), $resultSetPrototype);
                }, 

                //Event
                'Application\Model\RequestTable' =>  function($sm)
                {
                    $tableGateway = $sm->get('RequestGateway');
                    $table = new RequestTable($tableGateway);
                    return $table;
                },
                
                'RequestGateway' => function($sm)
                {
                    $dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
                    $resultSetPrototype = new ResultSet();
                    
                    return new TableGateway('va_request', $dbAdapter, new Feature\RowGatewayFeature('va_request_id'), $resultSetPrototype);
                },

                //Users
                'Application\Model\UsersTable' =>  function($sm)
                {
                    $tableGateway = $sm->get('UsersTableGateway');
                    $table = new UsersTable($tableGateway,$sm->get('config'));
                    return $table;
                },
                
                'UsersTableGateway' => function($sm)
                {
                    $dbAdapter = $sm->get('Zend\Db\Adapter\Adapter');
                    $resultSetPrototype = new ResultSet();
                    
                    return new TableGateway('va_users', $dbAdapter, new Feature\RowGatewayFeature('va_users_id'), $resultSetPrototype);
                },
			),
		);
	}
	
}
