<?php
/**
 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/ZendSkeletonApplication for the canonical source repository
 * @copyright Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License
 */

namespace Application\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\View\Model\JsonModel;
use Zend\Session\Container;
use Zend\Http\Client;
use Zend\Http\Request;
use Zend\Json\Json;
use Zend\Debug\Debug;
use WSClientLib\WSClient;

class ApiController extends AbstractActionController
{	
	private $config = null;

    public function _config(){
        if(is_null($this->config)){
            $this->config = json_decode(json_encode($this->getServiceLocator()->get('Config')));
        }
        return $this->config;
    }

    /**
    * Obtener lista de ciudades
    * @return JSON
    */
    public function getCitiesAction(){
    	$reqParams = $this->getRequest()->getQuery();

    	//$validation = $this->Tools()->validateApiKey($reqParams);

    	//if($validation['response']){
    		$arrJson = array(
				'response' => true,
                'msg'  => 'Lista de ciudades',
				'data' => $this->getServiceLocator()
					->get('Application/Model/Dbquery')
					->getCitiesList()
			);
    	/*}
    	else{
    		$arrJson = array(
				'response' => false,
				'msg' => $validation['message']
			);
    	}*/
		
		echo \Zend\Json\Json::encode($arrJson);

    	return $this->getResponse();
    }


	/**
    * Registrar customer
    * @param POST data
    * @return JSON
    */
    public function registerAction(){
    	$reqPost = $this->getRequest()->getPost();
    	
    	$_customer = $this->getServiceLocator()->get('Application/Model/CustomerTable');

    	$errors = array();
    	if(!isset($reqPost["cityId"]) || $reqPost["cityId"] == ''){
    		$errors[] = "cityId requerido";
    	}
    	if(!isset($reqPost["name"]) || $reqPost["name"] == ''){
    		$errors[] = "name requerido";
    	}
    	if(!isset($reqPost["lastname"]) || $reqPost["lastname"] == ''){
    		$errors[] = "lastname requerido";
    	}
    	if(!isset($reqPost["secondLastname"]) || $reqPost["secondLastname"] == ''){
    		$errors[] = "secondLastname requerido";
    	}
    	if(!isset($reqPost["email"]) || $reqPost["email"] == ''){
    		$errors[] = "email requerido";
    	}
    	if(!isset($reqPost["phone"]) || $reqPost["phone"] == ''){
    		$errors[] = "phone requerido";
    	}
    	if(!isset($reqPost["password"]) || $reqPost["password"] == ''){
    		$errors[] = "password requerido";
    	}
    		
		if(count($errors) == 0){//No hay errores
			//Validar si ya existe el customer en la BD
			if($_customer->customerExist($reqPost['email'])){
    			$arrJson = array(
					'response' => false,
					'msg' => "Ya hay una cuenta asociada a este email: " . $reqPost['email'],
                    'data' => null
				);
    		}
    		else{
    			//Crear nuevo customer
    			$data = array();
				$data['va_city_id']                 = $reqPost['cityId'];
				$data['va_customer_name']           = $reqPost['name'];
				$data['va_customer_lastname']       = $reqPost['lastname'];
				$data['va_customer_secondlastname'] = $reqPost['secondLastname'];
				$data['va_customer_email']          = $reqPost['email'];
				$data['va_customer_phone']          = $reqPost['phone'];
				$data['va_customer_pass']           = md5($reqPost['password'] . $this->_config()->pass->concat );
				$data['va_customer_code']           = md5($reqPost['email']);

    			$customerId = $_customer->save($data);
    			
    			if($customerId){
    				$arrJson = array(
						'response' => true,
						'msg' => 'Cliente '.$customerId.' registrado correctamente',
                        'data' => null
					);
    			}
    			else{
    				$arrJson = array(
						'response' => false,
						'msg' => 'Hubo un problema al registrar al cliente. Inténtalo más tarde',
                        'data' => null
					);
    			}
    		}
    	}
    	else{
    		$arrJson = array(
				'response' => false,
				'msg' => implode(', ', $errors),
                'data' => null
			);
    	}
    	
		echo \Zend\Json\Json::encode($arrJson);

    	return $this->getResponse();
    }


    /**
    * Inicio de sesión
    * @param POST email, password 
    * @return JSON
    */
    public function loginAction(){
    	$reqPost = $this->getRequest()->getPost();
    	
    	$_customer = $this->getServiceLocator()->get('Application/Model/CustomerTable');

    	$errors = array();
    	if(!isset($reqPost["email"]) || $reqPost["email"] == ''){
    		$errors[] = "email requerido";
    	}
    	if(!isset($reqPost["password"]) || $reqPost["password"] == ''){
    		$errors[] = "password requerido";
    	}
    	
    		
		if(count($errors) == 0){//No hay errores
			//Validar si ya existe el customer en la BD
			$customerId = $_customer
				->login($reqPost['email'], md5($reqPost['password'] . $this->_config()->pass->concat));

			if($customerId){
				$customer = $_customer->getCustomerById($customerId);

				$customerData = array(
					"customerId"             => $customer['va_customer_id'],
					"customerName"           => $customer['va_customer_name'],
					"customerLastname"       => $customer['va_customer_lastname'],
					"customerSecondLastname" => $customer['va_customer_secondlastname'],
					"customerEmail"          => $customer['va_customer_email'],
					"customerPhone"          => $customer['va_customer_phone'],
					"customerCode"           => $customer['va_customer_code']
				);

    			$arrJson = array(
					'response' => true,
                    'msg' => 'Inicio de sesión correcto',
					'data' => $customerData
				);
    		}
    		else{
				$arrJson = array(
					'response' => false,
					'msg' => 'Email o Password incorrectos',
                    'data' => null
				);
    		}
    	}
    	else{
    		$arrJson = array(
				'response' => false,
				'msg' => implode(', ', $errors),
                'data' => null
			);
    	}
    	
		echo \Zend\Json\Json::encode($arrJson);

    	return $this->getResponse();
    }


    /**
    * Registrar un nuevo va_request
    * @return JSON
    */
    public function createRequestAction(){
        $reqParams = $this->getRequest()->getPost();
        $validation = $this->Tools()->validateApiKey($reqParams);

        if($validation['response']){
            $_request = $this->getServiceLocator()->get('Application/Model/RequestTable');
            $errors = array();

            if(!isset($reqParams["customerId"]) || $reqParams["customerId"] == ''){
                $errors[] = "customerId requerido";
            }
            if(!isset($reqParams["appId"]) || $reqParams["appId"] == ''){
                $errors[] = "appId requerido";
            }
            if(!isset($reqParams["appsTotal"]) || $reqParams["appsTotal"] == ''){
                $errors[] = "appsTotal requerido";
            }
            if(!isset($reqParams["cost"]) || $reqParams["cost"] == ''){
                $errors[] = "cost requerido";
            }
            if(!isset($reqParams["currentLat"]) || $reqParams["currentLat"] == ''){
                $errors[] = "currentLat requerido";
            }
            if(!isset($reqParams["currentLng"]) || $reqParams["currentLng"] == ''){
                $errors[] = "currentLng requerido";
            }
            if(!isset($reqParams["originLat"]) || $reqParams["originLat"] == ''){
                $errors[] = "originLat requerido";
            }
            if(!isset($reqParams["originLng"]) || $reqParams["originLng"] == ''){
                $errors[] = "originLng requerido";
            }
            if(!isset($reqParams["destinyLat"]) || $reqParams["destinyLat"] == ''){
                $errors[] = "destinyLat requerido";
            }
            if(!isset($reqParams["destinyLng"]) || $reqParams["destinyLng"] == ''){
                $errors[] = "destinyLng requerido";
            }
            
                
            if(count($errors) == 0){//No hay errores
               //Crear nuevo event
                $data = array();
                $data["va_customer_id"]             = $reqParams['customerId'];
                $data["va_vehicletype_id"]          = $reqParams['appId'];
                $data["va_requeststatus_id"]        = 4;//No aceptado
                $data["va_request_appstotal"]       = $reqParams['appsTotal'];
                $data["va_request_cost"]            = $reqParams['cost'];
                $data["va_request_currentlat"]      = $reqParams['currentLat'];
                $data["va_request_currentlng"]      = $reqParams['currentLng'];
                $data["va_request_originlat"]       = $reqParams['originLat'];
                $data["va_request_originlng"]       = $reqParams['originLng'];
                $data["va_request_origindistance"]  = (isset($reqParams['originDistance'])) ? $reqParams['originDistance'] : NULL; 
                $data["va_request_destinylat"]      = $reqParams['destinyLat'];
                $data["va_request_destinylng"]      = $reqParams['destinyLng'];
                $data["va_request_destinydistance"] = (isset($reqParams['destinyDistance'])) ? $reqParams['destinyDistance'] : NULL; 

                $requestId = $_request->save($data);
                
                if($requestId){
                    $arrJson = array(
                        'response' => true,
                        'msg' => 'Request ' .$requestId. ' registrado correctamente',
                        'data' => array('requestId' => $requestId)
                    );
                }
                else{
                    $arrJson = array(
                        'response' => false,
                        'msg' => 'Hubo un problema. Inténtalo más tarde',
                        'data' => null
                    );
                }
            }
            else{
                $arrJson = array(
                    'response' => false,
                    'msg' => implode(', ', $errors),
                    'data' => null
                );
            }
        }
        else{
            $arrJson = array(
                'response' => false,
                'msg' => $validation['message']
            );
        }
        

        $varsToView["resultJson"] = json_encode($arrJson);

        //Retornar JSON
        $this->Tools()->preRenderView($varsToView, 'application/ajax/response');

        if($requestId){
            $this->Tools()->log('Entro al if', true, "ws_test");
            $this->Tools()->log($requestId, true, "ws_test");

            $_request = $this->getServiceLocator()->get('Application/Model/RequestTable');
            $dbquery = $this->getServiceLocator()->get('Application/Model/Dbquery');

            $request = $_request->getRequestById($requestId);

            if(is_array($request) && count($request) > 0){
                $this->Tools()->log("segundo if", true, "ws_test");

                $data = array();
                $data["module"]          = "request";
                $data["customerId"]      = "888";
                $data["event"]           = "newRequest";

                $data["requestId"]        = $requestId;
                $data["date"]             = $this->DateHelper()                                        
                    ->dateFormat($request["va_request_datetime"], "dayAndMonth");
                $data["time"]             = $this->DateHelper()
                    ->dateFormat($request["va_request_datetime"], "justTime");
                $data["cityName"]         = $request["va_city_name"];
                $data["customerFullname"] = $request["va_customer_name"] . " " 
                    . $request["va_customer_lastname"] . " "
                    . $request["va_customer_secondlastname"];
                $data["appsTotal"]        = $request["va_request_appstotal"];
                $data["appName"]          = $request["va_app_name"];
                $data["vehicleType"]      = $request["va_vehicletype_name"];
                $data["statusName"]       = "No Aceptado";
                $data["cost"]             = $request["va_request_cost"];                              

                $data = json_encode($data);

                $this->Tools()->log($data, true, "ws_test");
                
                $this->Tools()->log("ws://".$dbquery->WS_NOTIFICATIONS_ADDRESS.":".$dbquery->WS_NOTIFICATIONS_PORT."/ws?isPublisher=1&publisherToken=".$dbquery->WS_PUBLISHER_TOKEN."&publisherId=".$dbquery->WS_PUBLISHER_ID, true, "ws_test");

                $client = new WSClient("ws://".$dbquery->WS_NOTIFICATIONS_ADDRESS.":".$dbquery->WS_NOTIFICATIONS_PORT."/ws?isPublisher=1&publisherToken=".$dbquery->WS_PUBLISHER_TOKEN."&publisherId=".$dbquery->WS_PUBLISHER_ID);
                $this->Tools()->log('Despues de instanciar WSClient', true, "ws_test");

                $sendData = $client->send($data);

                $this->Tools()->log($sendData, true, "ws_test");
            }
        }
    }

    /**
    * Cambiar status del request
    * @param POST
    * @return JSON
    */
    public function updateStatusRequestAction(){
        $reqParams = $this->getRequest()->getPost();
        $validation = $this->Tools()->validateApiKey($reqParams);
        $arrJson = array('response' => false, 'msg' => '-');

        if($validation['response']){
            $_request = $this->getServiceLocator()->get('Application/Model/RequestTable');
            $errors = array();

            if(!isset($reqParams["requestId"]) || $reqParams["requestId"] == ''){
                $errors[] = "requestId requerido";
            }
            if(!isset($reqParams["statusId"]) || $reqParams["statusId"] == ''){
                $errors[] = "statusId requerido";
            }
    
            if(count($errors) == 0){//No hay errores
                $requestUpdate = 0;
                $set = array();
                /*
                1 Aceptado
                2 Iniciado
                3 Completado
                */
                switch ($reqParams['statusId']) {
                    case '1':
                        $set = array(
                            'va_requeststatus_id'     => $reqParams["statusId"],
                            'va_request_acceptedtime' => date('Y-m-d H:i:s')
                        );
                        break;
                    case '2':
                        $set = array(
                            'va_requeststatus_id'  => $reqParams["statusId"],
                            'va_request_starttime' => date('Y-m-d H:i:s')
                        );
                        break;
                    case '3':
                        $set = array(
                            'va_requeststatus_id'      => $reqParams["statusId"],
                            'va_request_completedtime' => date('Y-m-d H:i:s')
                        );
                        break;
                    default:
                        $set = array('va_requeststatus_id' => $reqParams["statusId"]);
                        break;
                }

                $requestUpdate = $_request->update($set, array('va_request_id' => $reqParams["requestId"]));
                       
                if($requestUpdate){
                    $arrJson = array(
                        'response' => true,
                        'msg' => 'Request ' .$reqParams["requestId"]. ', status actualizado',
                        'data' => null
                    );
                }
                else{
                    $arrJson = array(
                        'response' => false,
                        'msg' => 'No hubo cambios en la BD. El requestId no está en la BD o posiblemente estés notificando un status que ya se ha aplicado',
                        'data' => null
                    );
                }
            }
            else{
                $arrJson = array(
                    'response' => false,
                    'msg' => implode(', ', $errors),
                    'data' => null
                );
            }
        }
        else{
            $arrJson = array(
                'response' => false,
                'msg' => $validation['message']
            );
        }
        

        $varsToView["resultJson"] = json_encode($arrJson);

        //Retornar JSON
        $this->Tools()->preRenderView($varsToView, 'application/ajax/response');

        //WebSocket
        if($arrJson["response"]){
            $this->Tools()->log('request update status', true, "ws_test");

            $dbquery = $this->getServiceLocator()->get('Application/Model/Dbquery');
            $requestStatus = $dbquery->getRequestStatusById($reqParams['statusId']);
            
            $data = array();
            $data["module"]     = "request";
            $data["customerId"] = "888";
            $data["event"]      = "updateStatusRequest";
            $data["requestId"]  = $reqParams['requestId'];
            $data["statusId"]   = $reqParams['statusId'];
            $data["statusName"] = $requestStatus[0]['va_requeststatus_name'];
            $data = json_encode($data);

            $this->Tools()->log($data, true, "ws_test");
            
            $this->Tools()->log("ws://".$dbquery->WS_NOTIFICATIONS_ADDRESS.":".$dbquery->WS_NOTIFICATIONS_PORT."/ws?isPublisher=1&publisherToken=".$dbquery->WS_PUBLISHER_TOKEN."&publisherId=".$dbquery->WS_PUBLISHER_ID, true, "ws_test");

            $client = new WSClient("ws://".$dbquery->WS_NOTIFICATIONS_ADDRESS.":".$dbquery->WS_NOTIFICATIONS_PORT."/ws?isPublisher=1&publisherToken=".$dbquery->WS_PUBLISHER_TOKEN."&publisherId=".$dbquery->WS_PUBLISHER_ID);
            $this->Tools()->log('Despues de instanciar WSClient', true, "ws_test");

            $sendData = $client->send($data);

            $this->Tools()->log($sendData, true, "ws_test");
            
        }
    }

	
	public function testAction()
	{
        $dbquery = $this->getServiceLocator()->get('Application/Model/Dbquery');
		$requestStatus = $dbquery->getRequestStatusById(1);

        var_dump($requestStatus);
        return $this->getResponse();
	}
	
	
}
