<?php
/**

 * Zend Framework (http://framework.zend.com/)
 *
 * @link      http://github.com/zendframework/ZendSkeletonApplication for the canonical source repository
 * @copyright Copyright (c) 2005-2015 Zend Technologies USA Inc. (http://www.zend.com)
 * @license   http://framework.zend.com/license/new-bsd New BSD License

 */



namespace Application\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\View\Model\JsonModel;
use Zend\Session\Container;
use Zend\Debug\Debug;

class LandingController extends AbstractActionController

{

    private $config = null;

    public function _config(){

        if(is_null($this->config)){
            $this->config = json_decode(json_encode($this->getServiceLocator()->get('Config')));
        }

        return $this->config;
    }

	

	public function listAction(){

		$varsToView = array();
		$varsToView["dateHelper"] = $this->DateHelper();
		$reqParams = $this->getRequest()->getQuery();
		$userSessionAuth = new Container('userSessionAuth');

		$creation = $this->params()->fromRoute('slashParams');

		$varsToView["creation"] = $creation;

		//Debug::dump($creation);
		//die();

		if($userSessionAuth->offsetExists('loggedIn')){
			if(!$this->Tools()->hasAccess('landing_index')){
                return $this->redirect()->toRoute('application');
            }

			$this->layout('layout/default');
			$slashParams = $this->params()->fromRoute('slashParams');

			//Debug::dump($slashParams);

			if($slashParams){
				$assocSlashParams = $this->Tools()->getSlashParams($slashParams);
			}

			$landingModel = $this->getServiceLocator()->get('Application/Model/Landing');
			$varsToView['landingPagesList'] = $landingModel->getLandingPagesList();

			$this->layout()->setVariable('pageTitle', 'Landing Pages listado');
		}else{
			$varsToView["loggedIn"] = false;
			return $this->redirect()->toRoute('application');
		}

		$varsToView["config"] = $this->_config();
		$varsToView["tools"] = $this->Tools();
		$viewModel = new ViewModel($varsToView);

		return $viewModel;
	}

	

	public function formAction(){

		$varsToView = array();
		$varsToView["dateHelper"] = $this->DateHelper();
		$reqParams = $this->getRequest()->getQuery();
		$userSessionAuth = new Container('userSessionAuth');


		if($userSessionAuth->offsetExists('loggedIn')){
			$this->layout('layout/default');

			$slashParams = $this->params()->fromRoute('slashParams');
			//Debug::dump($slashParams);

			if($slashParams){
				$assocSlashParams = $this->Tools()->getSlashParams($slashParams);
			}

			if(isset($assocSlashParams["id"])){//Traemos la info de un registro para Actualizarlo
				if(!$this->Tools()->hasAccess('landing_update')){
				    return $this->redirect()->toRoute('application');
				}

				$landingModel = $this->getServiceLocator()->get('Application/Model/Landing');
				$varsToView['landingPage'] = $landingModel->getLandingPage($assocSlashParams["id"]);

				$this->layout()->setVariable('pageTitle', 'Editar Landing');

			}else{
				if(!$this->Tools()->hasAccess('landing_create')){
				    return $this->redirect()->toRoute('application');
				}

				$this->layout()->setVariable('pageTitle', 'Nuevo Landing');

			}

			

		}else{

			$varsToView["loggedIn"] = false;

			return $this->redirect()->toRoute('application');

		}

		$varsToView["config"] = $this->_config();

		$viewModel = new ViewModel($varsToView);

		return $viewModel;

		

	}

	

	

	public function createAction(){
		ini_set('display_errors', 1);
        ini_set('display_startup_errors', 1);
        error_reporting(E_ALL);

		$varsToView = array();
		$reqParams = $this->getRequest()->getQuery();
		$reqPost = $this->getRequest()->getPost();
		$userSessionAuth = new Container('userSessionAuth');
		$landingId = null;

		//Debug::dump($reqPost);

		if($userSessionAuth->offsetExists('loggedIn')){
			if(isset($reqPost["url"])){
				$landingModel = $this->getServiceLocator()->get('Application/Model/Landing');

				if(!isset($reqPost["id"]) || trim($reqPost["id"]) == ""){
					$newLanding = $landingModel->create($reqPost);
					$landingId = $newLanding;

					if($newLanding){//Registro correcto

						//return $this->redirect()->toRoute('application');
						if(count($_FILES) > 0){
							$this->uploadLandingImages($landingId, $_FILES);
						}
						
						return $this->redirect()->toRoute('Landing',
							array('controller'=>"Landing",
								'action' => "list",
								'creation' => "itemCreated"
							)

						);
						echo "Despues de redireccionar";
					}
				}elseif(isset($reqPost["id"])){
					$editedLanding = $landingModel->update($reqPost);
					$landingId = $reqPost["id"];

					if($editedLanding){//Registro actualizado
						if(count($_FILES) > 0){
							$this->uploadLandingImages($landingId, $_FILES);
						}

						return $this->redirect()->toRoute('Landing',
							array('controller'=>"Landing",
								'action' => "list",
								'creation' => "itemUpdated"
							)
						);
					}
				}

			}else{

			}
		}else{
			$varsToView["loggedIn"] = false;
			return $this->redirect()->toRoute('application');
		}

		$viewModel = new ViewModel($varsToView);

		return $viewModel;
	}

	public function uploadLandingImages($id, $files){
		if(count($files) > 0 && is_numeric($id)){
			$imagesData = array();
			foreach ($files as $key => $file) {
			    // Make certain to merge the files info!
			    
			    $allowedExts = array("gif", "jpeg", "jpg", "png");
			    $fileExploded = explode(".", $file["name"]);
			    $extension = end($fileExploded);
			    //echo "Ext:" . $extension;
			    $uploadDir = $this->_config()->server->BASE_PATH . "/public/img/landings/";
			    $md5Id = null;

			    //echo "ID:";
			    //var_dump($id);
			    if(isset($id) && is_numeric($id)){
			        $md5Id = md5($id);

			        if (!file_exists($uploadDir . $md5Id)) {
			            mkdir($uploadDir . $md5Id, 0777, true);
			        }
			        
			        $uploadDir .=  $md5Id . "/";
			    }

			    //echo "type" . $_FILES["files"]["type"][0];
			    if ((($file["type"] == "image/gif")
			        || ($file["type"] == "image/jpeg")
			        || ($file["type"] == "image/jpg")
			        || ($file["type"] == "image/pjpeg")
			        || ($file["type"] == "image/x-png")
			        || ($file["type"] == "image/png")
			        || ($file["type"] == "")//Algunos archivos de imagen no tienen esta key
			        )
			        //&& ($_FILES["files"]["size"][0] < 1000000)
			        && in_array($extension, $allowedExts)
			        && $file["name"] != ""
			        ){
			        if ($file["error"] > 0){
			            //echo "Return Code: " . $_FILES["files"]["error"][0] . "<br>";
			            //$response["files"][0]["message"] = "Return Code: " . $_FILES["files"]["error"][0];
			        }else{
			            
			            $movido = move_uploaded_file($file["tmp_name"], $uploadDir . $file["name"]);
			            
			            if($movido){
			                $response["files"][0]["status"] = true;

			                //Columna de BD
			                $imgUrl = "/". $md5Id ."/". $file["name"];
			                $imagesData["as_landingpage_" . $key] = $imgUrl;
			            
			            }else{
			                //$response["files"][0]["message"] = "Error al cargar archivo";
			            }
			            
			        }
			    }else{                    
			        //$response["files"][0]["message"] = "Archivo invalido";
			    }
			}//end foreach

			//Actualizar BD con imagenes
			if(count($imagesData) > 0){
				$landingModel = $this->getServiceLocator()->get('Application/Model/Landing');
				$editedImg = $landingModel->updateImages($imagesData, $id);

				//echo "editedImg Resp:<br>";
				//var_dump($editedImg);
			}
		}
	}

	

}

