<?php 
namespace Application\Model;


use Zend\Db\TableGateway\TableGateway;
use Zend\Db\Sql\Sql;
use Zend\Debug\Debug;


class UsersTable
{
	protected $tableGateway;
	private $config;
	private $now;

	//https://framework.zend.com/manual/2.0/en/user-guide/database-and-models.html
	//Aca la documentación para ejemplos
	public function __construct(TableGateway $tableGateway,$config)
	{
		$this->tableGateway = $tableGateway;
		$this->config = $config;
		$this->now = date("Y-m-d H:i:s");
	}


	public function fetchAll()
	{
		$resultSet = $this->tableGateway->select();
		return $resultSet;
	}
	
    

	public function getUser($users_id){
		$rowset = $this->tableGateway->select(array('va_users_id' => $users_id));
		$row = $rowset->current();
		if (!$row) {
			return false;
		}
		return $row;
	}
	
	public function getAllUsers(){
		
		$sql = "
			SELECT 
				u.va_users_id,
				u.va_users_fullname,
				u.va_users_email,
				u.va_users_status,
				r.va_roles_name
			FROM va_users u
			LEFT JOIN va_roles r ON u.va_roles_id = r.va_roles_id
            ";
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	public function getUsersByIdRole($idRole){
		
		$sql = "
			SELECT 
				u.va_users_id,
				u.va_users_fullname,
				r.va_roles_name
			FROM va_users u
			LEFT JOIN va_roles r ON u.va_roles_id = r.va_roles_id
			WHERE r.va_roles_status = 1 AND r.va_roles_id = " . $idRole ;
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	
	public function getActiveRoles(){
		$sql = "
			SELECT *
			FROM va_roles
			WHERE va_roles_status = 1";
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	public function getRoleById($role_id){
		$sql = "
			SELECT *
			FROM va_roles
			WHERE va_roles_id = " . $role_id;
		$rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();		
	}
	
	public function getRoles(){
		$sql = "
			SELECT *
			FROM va_roles";
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	public function getRolePermissionsById($role_id){
		$sql = "
			SELECT va_rules_id
			FROM va_permissions
			WHERE va_roles_id = " . $role_id . "
			AND va_permissions_status = 1";
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	public function getRules(){
		$sql = "
			SELECT *
			FROM va_rules
			WHERE va_rules_status = 1
			ORDER BY va_rules_str ASC";
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
	}
	
	public function setUser($_data)
	{
		/*$data = array(
			'va_roles_id'			=> $_data["va_roles_id"],
			'va_users_fullname'		=> $_data["va_users_fullname"],
			'va_users_email'		=> $_data["va_users_email"],
			'va_users_status'		=> $_data["va_users_status"]
		);*/
		
		
		

		if(isset($_data["va_users_password"]) && $_data["va_users_password"]!==''){
			$_data["va_users_password"] = md5($_data["va_users_password"].$this->config['pass']['concat']);
		}
        
		
		
		
		if (!isset($_data["va_users_id"])) {
			$this->tableGateway->insert($_data);
			return $this->tableGateway->lastInsertValue;
		} else {
			$result = $this->tableGateway->update($_data, array('va_users_id' => $_data["va_users_id"]));
			//var_dump($data);
			return $result;
		}
		
	}

	public function setRole($role_name=''){
		
		$sql = new Sql($this->tableGateway->getAdapter());
		$insert = $sql->insert('va_roles');
		
		$newData = array(
			'va_roles_id'=> null,
			'va_roles_name'=> $role_name,
			'va_roles_updated'=> $this->now,
			'va_roles_status'=> 1
		);
		
		$insert->values($newData);
		$selectString = $sql->getSqlStringForSqlObject($insert);
		
		
		$result = $this->tableGateway->getAdapter()->driver->getConnection()->execute($selectString);
		
		return $result;
	}
	
	public function disableEntireRolePermissionsById($roles_id){
		
		$sql = new Sql($this->tableGateway->getAdapter());
		$update = $sql->update('va_permissions');
		$newData = array(
			'va_permissions_status'	=> 0
		);
		$update->set($newData);
		$update->where('va_roles_id = '.$roles_id.' AND va_permissions_status = 1');
		$updateString = $sql->getSqlStringForSqlObject($update);
		
		$result = $this->tableGateway->getAdapter()->driver->getConnection()->execute($updateString);
		
		return $result;
		
	}
	
	public function updateRole($role_id, $role_name='', $role_status=1){
		
		$sql = new Sql($this->tableGateway->getAdapter());
		$update = $sql->update('va_roles');
		$newData = array(
			'va_roles_name'=> $role_name,
			'va_roles_status'=> $role_status
			
		);
		$update->set($newData);
		$update->where('va_roles_id = '.$role_id);
		$updateString = $sql->getSqlStringForSqlObject($update);
		$result = $this->tableGateway->getAdapter()->driver->getConnection()->execute($updateString);
		
		return $result;
		
	}
	
	public function setRolePermission($role_id, $rule_id){
		
		$sql = new Sql($this->tableGateway->getAdapter());
		$insert = $sql->insert('va_permissions');
		
		$newData = array(
			'va_permissions_id'=> null,
			'va_roles_id'=> $role_id,
			'va_rules_id'=> $rule_id, 
			'va_permissions_updated' => $this->now,
			'va_permissions_status' => 1
			
		);
		
		$insert->values($newData);
		$insertString = $sql->getSqlStringForSqlObject($insert);
		
		//Debug::dump($insertString);
		
		$result = $this->tableGateway->getAdapter()->driver->getConnection()->execute($insertString);
		
		return $result;
		
	}
	
	public function getUserPermissions($users_id){
		
		$sql = "
			SELECT r.va_rules_str
			FROM va_rules r
			JOIN va_permissions p ON r.va_rules_id = p.va_rules_id AND p.va_permissions_status = 1
			JOIN va_users u ON p.va_roles_id = u.va_roles_id
			WHERE r.va_rules_status = 1
			AND u.va_users_id = " . $users_id;
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
		
	}
	
	public function getUserPermissionsApp($users_id){
		
		$sql = "
			SELECT 
				r.va_rules_str AS permissions
			FROM va_rules r
			JOIN va_permissions p ON r.va_rules_id = p.va_rules_id AND p.va_permissions_status = 1
			JOIN va_users u ON p.va_roles_id = u.va_roles_id
			WHERE r.va_rules_status = 1
			AND (r.va_rules_str LIKE 'app%' OR r.va_rules_str LIKE 'todo' OR r.va_rules_str LIKE 'nada')
			AND u.va_users_id = " . $users_id;
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
		
	}
	
	
	public function getRoleByUserId($users_id){
		
		$sql = "
			SELECT 
				ro.va_roles_name AS role,
				ro.va_roles_id AS roleId
			FROM
				va_roles ro
			JOIN va_users u ON u.va_roles_id = ro.va_roles_id
			WHERE u.va_users_id =  " . $users_id;
		
        $rs = $this->tableGateway->getAdapter()->driver->getConnection()->execute($sql);
        return $rs->allData();
		
	}

}

