<?php
namespace Application\Plugin;
 
use Zend\Mvc\Controller\Plugin\AbstractPlugin;
use Zend\Session\Container;
use Zend\Debug\Debug;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\ViewModel;
use Zend\View\Renderer\PhpRenderer;
use Zend\View\Resolver;
use Zend\Mail;
use Zend\xmlrpc;
use zendxml;
use Zend\Mime\Part as MimePart;
use Zend\Mime\Message as MimeMessage;
//use SimpleXMLElement;
use DOMDocument;
use XSLTProcessor;
use DateTime;


class DateHelper extends AbstractPlugin{
private $shortDays = array();
	private $shortMonths = array();
	
	private $days = array();
	private $months = array();
	
	
	function __construct(){
		
		date_default_timezone_set('America/Mexico_City');
		
		$this->shortDays["Sunday"] = "DOM";
		$this->shortDays["Monday"] = "LUN";
		$this->shortDays["Tuesday"] = "MAR";
		$this->shortDays["Wednesday"] = "MIE";
		$this->shortDays["Thursday"] = "JUE";
		$this->shortDays["Friday"] = "VIE";
		$this->shortDays["Saturday"] = "SAB";
		
		$this->shortMonths["January"] = "ENE";
		$this->shortMonths["February"] = "FEB";
		$this->shortMonths["March"] = "MAR";
		$this->shortMonths["April"] = "ABR";
		$this->shortMonths["May"] = "MAY";
		$this->shortMonths["June"] = "JUN";
		$this->shortMonths["July"] = "JUL";
		$this->shortMonths["August"] = "AGO";
		$this->shortMonths["September"] = "SEP";
		$this->shortMonths["October"] = "OCT";
		$this->shortMonths["November"] = "NOV";
		$this->shortMonths["December"] = "DIC";
		
		
		$this->days["Sunday"] = "DOMINGO";
		$this->days["Monday"] = "LUNES";
		$this->days["Tuesday"] = "MARTES";
		$this->days["Wednesday"] = "MIERCOLES";
		$this->days["Thursday"] = "JUEVES";
		$this->days["Friday"] = "VIERNES";
		$this->days["Saturday"] = "SABADO";
		
		$this->months["January"] = "ENERO";
		$this->months["February"] = "FEBRERO";
		$this->months["March"] = "MARZO";
		$this->months["April"] = "ABRIL";
		$this->months["May"] = "MAYO";
		$this->months["June"] = "JUNIO";
		$this->months["July"] = "JULIO";
		$this->months["August"] = "AGOSTO";
		$this->months["September"] = "SEPTIEMBRE";
		$this->months["October"] = "OCTUBRE";
		$this->months["November"] = "NOVIEMBRE";
		$this->months["December"] = "DICIEMBRE";
		
	}
	
	function getDeliveryDatesList($numDays=0, $startDate=""){
		
		if($startDate=="")	$startDate=date("Y-m-d");
		$arrayReturn = array();
		
		if($numDays===Null || !is_numeric($numDays))	return $arrayReturn;
		
		$weekendDays = explode(',', Mage::getStoreConfig('general/locale/weekend'));
		
		$incrementalDay=0;
		$indexReturn=0;
		while($indexReturn <= $numDays){
			$date = date("Y-m-d", strtotime($startDate." + $incrementalDay day"));
			
			$dayName = date_format(date_create($date), 'l');
			
			$numberDayOfWeek = date('w', strtotime($dayName));
			//echo "<br><br> Numero: ---".$dayName."--->".$numberDayOfWeek;
			
			if(in_array($numberDayOfWeek, $weekendDays)){
				$incrementalDay++;
				continue;
				//echo "<br>SI es fin de semana";
			}else{
				
				//echo "<br>No es fin de semana";
				
				//Ej: Lun
				$dayNameSpanish = strtr($dayName, $this->shortDays);
				$arrayReturn[$indexReturn]["text1"] = ($date == date("Y-m-d")?"HOY":$dayNameSpanish);
				
				//Ej: Nov 3
				$monthDayNum = date_format(date_create($date), 'F j');
				$monthDayNumSpanish = strtr($monthDayNum, $this->shortMonths);
				$arrayReturn[$indexReturn]["text2"] = ($date == date("Y-m-d")?"":$monthDayNumSpanish);
				
				//Ej: 2015-11-03
				$arrayReturn[$indexReturn]["date"] = $date;
				
				$incrementalDay++;
				$indexReturn++;
			}
		}
		
		return $arrayReturn;
		
	}
	
	/*
	* 
	* Calculate the midle between two times given
	* 
	* Params $time1 string with the format "09:00:00"
	* Params $time2 string with the format "10:00:00"
	* Return string with the format "09:30:00"
	*/
	function getMidleTime($time1="08:00:00", $time2="09:00:00"){
		
		
		$datetime1 = new DateTime($time1);
		$datetime2 = new DateTime($time2);
		$interval = $datetime1->diff($datetime2);
		$hours = $interval->format('%h');
		$middleInMinutes = (($hours/2)*60);
		
		return date("H:i:s", strtotime($time1." + $middleInMinutes minute"));
		
	}
	
	/*
	* 
	* Sum times and calculate if is possible deliver
	* 
	* Params $minsToSum Minutes to sum to Time From
	* Params $timeFrom Time allowed to choose from
	* Return bolean
	*/
	function isInTimeToDeliver($minsToSum=30, $timeFrom="2016-05-12 08:00:00"){
		
		$currentTime = date("Y-m-d H:i:s");
		$newTimeFrom = date("Y-m-d H:i:s", strtotime($timeFrom." - $minsToSum minute"));
		if(DateTime::createFromFormat('Y-m-d H:i:s', $currentTime)  > DateTime::createFromFormat('Y-m-d H:i:s', $newTimeFrom)){
			return false;
		}else{
			return true;
		}
		
	}

	/*
	** Retorna el tiempo restante que se tiene para realizar una entrega en minutos
	*  @params String $timeTo // Fecha y hora tope
	*  @return array // Minutos or Array con años, meses, dias, munutos, segundos
	*/
	public function remainTimeToDeliver($timeTo="2016-05-12 10:00:00", $returnAsArray=false){
		$currentTime = date("Y-m-d H:i:s");
		$start_date = new DateTime($currentTime);
		$end_date = new DateTime($timeTo);
		$since_start = $start_date->diff($end_date);
		$minutes = 0;
		$sign = ($start_date > $end_date) ? "-" : "";
		$minutes += $since_start->h * 60;
		$minutes += $since_start->i; 

		if($returnAsArray){
			$array = array();
			$array['y'] = $since_start->y;
			$array['m'] = $since_start->m;
			$array['d'] = $since_start->d;
			$array['h'] = $since_start->h;
			$array['i'] = sprintf("%02d", $since_start->i);
			$array['s'] = $since_start->s;
			$array['sign'] = $sign;
			return $array;
		}
		
		return array('minutes' => $minutes, 'sign' => $sign);
	}

	public function substractDates($timeFrom='', $timeTo="2016-05-12 10:00:00", $returnAsArray=false){
		$currentTime = $timeFrom;
		$start_date = new DateTime($currentTime);
		$end_date = new DateTime($timeTo);
		$since_start = $start_date->diff($end_date);
		$minutes = 0;
		$sign = ($start_date > $end_date) ? "-" : "";
		$minutes += $since_start->h * 60;
		$minutes += $since_start->i; 

		if($returnAsArray){
			$array = array();
			$array['y'] = $since_start->y;
			$array['m'] = $since_start->m;
			$array['d'] = $since_start->d;
			$array['h'] = $since_start->h;
			$array['i'] = sprintf("%02d", $since_start->i);
			$array['s'] = $since_start->s;
			$array['sign'] = $sign;
			return $array;
		}
		
		return array('minutes' => $minutes, 'sign' => $sign);
	}
	
	function isTimeInRange($timeToSearch, $timeFrom, $timeTo){
		
		$date1 = DateTime::createFromFormat('H:i:s', $timeToSearch);
		$date2 = DateTime::createFromFormat('H:i:s', $timeFrom);
		$date3 = DateTime::createFromFormat('H:i:s', $timeTo);
		if ($date1 > $date2 && $date1 < $date3){
			return true;
		}else{
			return false;
		}
		
	}
	
	function isAllowedDay($dateToCheck="", $arrayDaysAcl=array()){
		if($dateToCheck == "" || count($arrayDaysAcl) == 0)	return false;
		
		$dayName = date_format(date_create($dateToCheck), 'l');
		
		if($dayName == "Sunday" && $arrayDaysAcl["flo_horarios_sunday"] == 1){
			return true;
		}else if($dayName == "Monday" && $arrayDaysAcl["flo_horarios_monday"] == 1){
			return true;
		}else if($dayName == "Tuesday" && $arrayDaysAcl["flo_horarios_tuesday"] == 1){
			return true;
		}else if($dayName == "Wednesday" && $arrayDaysAcl["flo_horarios_wednesday"] == 1){
			return true;
		}else if($dayName == "Thursday" && $arrayDaysAcl["flo_horarios_thursday"] == 1){
			return true;
		}else if($dayName == "Friday" && $arrayDaysAcl["flo_horarios_friday"] == 1){
			return true;
		}else if($dayName == "Saturday" && $arrayDaysAcl["flo_horarios_saturday"] == 1){
			return true;
		}else{
			return false;
		}
		
	}
	
	function getCalendarData($monthNum=Null, $year=Null){
		date_default_timezone_set('America/Mexico_City');
		if($monthNum==Null)	$monthNum = date("m");
		if($year==Null)	$year = date("Y");
		$currentDayNum = date("j");
		$numDaysInMonth = cal_days_in_month(CAL_GREGORIAN, $monthNum, $year);
		
		$dateObj   = DateTime::createFromFormat('!m', $monthNum);
		$monthName = $dateObj->format('F'); // March
		
		$numDayInWeekFirstDayMonth = date_format(date_create(date($year."-".$monthNum."-01")), 'N');
		$numDayInWeekLastDayMonth = date_format(date_create(date($year."-".$monthNum."-".$numDaysInMonth)), 'N');
		
		if($numDayInWeekFirstDayMonth==1){
			$blankSpaceBefore = 0;//Lunes
		}else if($numDayInWeekFirstDayMonth==2){
			$blankSpaceBefore = 1;//Martes
		}else if($numDayInWeekFirstDayMonth==3){
			$blankSpaceBefore = 2;//Miercoles
		}else if($numDayInWeekFirstDayMonth==4){
			$blankSpaceBefore = 3;//Jueves
		}else if($numDayInWeekFirstDayMonth==5){
			$blankSpaceBefore = 4;//Viernes
		}else if($numDayInWeekFirstDayMonth==6){
			$blankSpaceBefore = 5;//Sabado
		}else if($numDayInWeekFirstDayMonth==7){
			$blankSpaceBefore = 6;//Domingo
		}
		
		
		if($numDayInWeekLastDayMonth==1){
			$blankSpaceAfter = 6;//
		}else if($numDayInWeekLastDayMonth==2){
			$blankSpaceAfter = 5;//
		}else if($numDayInWeekLastDayMonth==3){
			$blankSpaceAfter = 4;
		}else if($numDayInWeekLastDayMonth==4){
			$blankSpaceAfter = 3;
		}else if($numDayInWeekLastDayMonth==5){
			$blankSpaceAfter = 2;
		}else if($numDayInWeekLastDayMonth==6){
			$blankSpaceAfter = 1;
		}else if($numDayInWeekLastDayMonth==7){
			$blankSpaceAfter = 0;
		}
		
		$days = array();
		for($i=1;$i<=$numDaysInMonth;$i++){
			$date = $year."-".$monthNum."-".($i<10?"0".$i:$i);
			$days[$i]["dayNum"] = $i;
			$days[$i]["dayName"] = strtr(date('l', strtotime($year."-".$monthNum."-".$i)), $this->days);
			$days[$i]["date"] = $date;
			$days[$i]["today"] = ($date == date("Y-m-d")?1:0);
			$days[$i]["tomorrow"] = ($date == date("Y-m-d", strtotime("+ 1 day"))?1:0);
			
		}
		
		
		
		return array(
			"blankSpaceBefore" => $blankSpaceBefore,
			"blankSpaceAfter" => $blankSpaceAfter,
			"numDaysInMonth" => $numDaysInMonth,
			"monthNum" => $monthNum,
			"currentDayNum" => $currentDayNum,
			"shortMonthName" => strtr($monthName, $this->shortMonths),
			"monthName" => strtr($monthName, $this->months),
			"year" => $year,
			"days" => $days
		
		);
		
	}
	
	function dateFormat($datestring, $type = "full"){
		
		$date = date_create($datestring);
		
		$formated = null;
		//$formated = date_format($date, 'g:ia \o\n l jS F Y');
		//output: 5:45pm on Saturday 24th March 2012
		
		$days = array();
		$days["Sunday"] = "Domingo";
		$days["Monday"] = "Lunes";
		$days["Tuesday"] = "Martes";
		$days["Wednesday"] = "Miercoles";
		$days["Thursday"] = "Jueves";
		$days["Friday"] = "Viernes";
		$days["Saturday"] = "Sabado";

		//3 letters days
		$days["Sun"] = "Dom";
		$days["Mon"] = "Lun";
		$days["Tue"] = "Mar";
		$days["Wed"] = "Mié";
		$days["Thu"] = "Jue";
		$days["Fri"] = "Vie";
		$days["Sat"] = "Sáb";
		/*
		$months = array();
		$months["January"] = "Enero";
		$months["February"] = "Febrero";
		$months["March"] = "Marzo";
		$months["April"] = "Abril";
		$months["May"] = "Mayo";
		$months["June"] = "Junio";
		$months["July"] = "Julio";
		$months["August"] = "Agosto";
		$months["September"] = "Septiembre";
		$months["October"] = "Octubre";
		$months["November"] = "Noviembre";
		$months["December"] = "Diciembre";
		*/
		$months = array();
		$months["January"] = "Ene";
		$months["February"] = "Feb";
		$months["March"] = "Mar";
		$months["April"] = "Abr";
		$months["May"] = "May";
		$months["June"] = "Jun";
		$months["July"] = "Jul";
		$months["August"] = "Ago";
		$months["September"] = "Sep";
		$months["October"] = "Oct";
		$months["November"] = "Nov";
		$months["December"] = "Dic";
		
		if($type == "short"){
			$formated = date_format($date, 'F j, Y');
			//output:     March 24 de 2012
		}elseif($type == "dayAndMonth"){
			$formated = date_format($date, 'j F');
			//output:     March 24 de 2012
		}else if($type == "shorttime"){
			$formated = date_format($date, 'F j, Y, g:i A');
			//output:     March 24 de 2012
		}else if($type == "full"){
			$formated = date_format($date, ' l j \d\e F \d\e Y, g:i a');
			//output:     Saturday de 24 de March de 2012, 5:45pm
		}else if($type == "medium"){
			$formated = date_format($date, 'j \d\e F \d\e Y');
			//output:      24 de March de 2012
		}else if($type == "semiFull"){
			$formated = date_format($date, 'l j \d\e F \d\e Y');
			//output:      24 de March de 2012
		}else if($type == "justTime"){
			$formated = date_format($date, 'g:i A');
			//output: 7:45 PM
		}else if($type == "shortWithTime"){
			$formated = date_format($date, 'j/m/Y g:i A');
			//output: 30/05/2015 7:45 PM
		}else if($type == "time12h"){
			$formated = date_format($date, 'h:i');
			//output: 7:45 PM
		}else if($type == "time24h"){
			$formated = date_format($date, 'H:i');
			//output: 7:45 PM
		}
		else if($type == "time12hmeridiano"){
			$formated = date_format($date, 'A');
			//output: 7:45 PM
		}else if($type == "monthday"){
			$formated = date_format($date, 'F j');
			//output: March 24
		}else if($type == "3monthday"){
			$formated = date_format($date, 'M d');
			//output: March 24
		}else if($type == "3dayofweek"){
			$formated = date_format($date, 'D');
			//output: March 24
		}else if($type == "dayofweek"){
			$formated = date_format($date, 'l');
			//output: March 24
		}
		$formated = strtr($formated, $days);
		$formated = strtr($formated, $months);
		return $formated;
		
	}
	
	/*
	* 
	* Calculate the product disponibility
	* 
	* Params $productId string numeric or int with the product ID "15" or 15
	* Return string with the disponibility "Disponible hoy 5PM - 6PM"
	*/
	function getProductDisponibilityById($productId=null, $dateToCheck=null){
		date_default_timezone_set('America/Mexico_City');
		if($productId==Null || $productId=="" || !is_numeric($productId))	return "";
		$today = date("Y-m-d");
		$dateToCheck = $dateToCheck==null?date("Y-m-d"):$dateToCheck;
		$productsCanDeliverySameDay = true;
		
		//Checamos si es dia inhabil, fin de semana
		$dayName = date_format(date_create($dateToCheck), 'l');
		$numberDayOfWeek = date('w', strtotime($dayName));
		$weekendDays = explode(',', Mage::getStoreConfig('general/locale/weekend'));
		if(in_array($numberDayOfWeek, $weekendDays)){
			return $this->getProductDisponibilityById($productId, date("Y-m-d", strtotime($dateToCheck." + 1 day")));
		}
		
		$product = Mage::getModel('catalog/product')->load($productId);
		if($product->getResource()->getAttribute('mismo_dia')->getFrontend()->getValue($product) == 'No'){
			$productsCanDeliverySameDay = false;
		}
		
		if($today == $dateToCheck){
			$dayText = "hoy";
		}else if($dateToCheck == date("Y-m-d", strtotime($today." + 1 day"))){
			$dayText = "mañana";
		}else{
			$dayText = $this->dateFormat($dateToCheck, "monthday");
		}
		
		$restriccionesModel = Mage::getSingleton('calendario/restricciones');
		$restriccionesByDate = $restriccionesModel->getRestriccionesByDate($dateToCheck);
		
		$horariosModel = Mage::getSingleton('calendario/horarios');
		$deliveries = $horariosModel->getDeliveriesListByDate($dateToCheck);
		
		if($restriccionesByDate && count($restriccionesByDate) > 0){
			//Existen restricciones de horario para hoy
			
			if($restriccionesByDate[0]["flo_calendario_restriccionestipo_id"] == 1){
				//Checamos si es dia inhabil, por festivo 
				return $this->getProductDisponibilityById($productId, date("Y-m-d", strtotime($dateToCheck." + 1 day")));
			}
			
			$horariosRestriccionesModel = Mage::getSingleton('calendario/horariorestricciones');
			$horarioRestricciones = $horariosRestriccionesModel->getHorarioRestriccionesByRestriccionesId($restriccionesByDate[0]["flo_calendario_restricciones_id"]);
			for($i=0; $i < count($horarioRestricciones); $i++){
				
				//Checamos cuantas entregas estan dentro de los rangos de horarios a mosrar
				$numDeliveriesInHorario = 0;
				for($e=0;$e<count($deliveries);$e++){
					if($this->isTimeInRange($deliveries[$e]["deliverytimemiddle"], $horarioRestricciones[$i]["flo_horario_restricciones_timefrom"], $horarioRestricciones[$i]["flo_horario_restricciones_timeto"])){
						$numDeliveriesInHorario++;
					}
				}
				
				$cupo = ($horarioRestricciones[$i]["flo_horario_restricciones_deliverylimit"] - $numDeliveriesInHorario);
				//Checamos si esta en tiempo para entrega mismo dia
				if($today == $dateToCheck){
					if(!$this->isInTimeToDeliver($horarioRestricciones[$i]["flo_horario_restricciones_buytimebeforeshipment"], $horarioRestricciones[$i]["flo_horario_restricciones_timefrom"])){
						$cupo = 0;
					}
					
					if(!$productsCanDeliverySameDay){
						$cupo = 0;
					}
					
				}
				
				if($cupo > 0){
					$text = "Disponible ";
					$text .= $dayText." ".$horarioRestricciones[$i]["flo_horario_restricciones_timefrom_int"]."".$horarioRestricciones[$i]["flo_horario_restricciones_timefrom_m"];
					$text .= " - ".$horarioRestricciones[$i]["flo_horario_restricciones_timeto_int"]."".$horarioRestricciones[$i]["flo_horario_restricciones_timeto_m"];
					return $text;
				}
					
				
				
			}
			
			//Termino el bucle de horarios y no hubo cupo en ninguno
			return $this->getProductDisponibilityById($productId, date("Y-m-d", strtotime($dateToCheck." + 1 day")));
			
			
		}else{
			$horariosDefault = $horariosModel->getHorariosDefault();
			//Zend_Debug::dump($horariosDefault, $label = "<br />:::Horarios Default:::");
			for($i=0; $i < count($horariosDefault); $i++){
				if($this->isAllowedDay($dateToCheck, $horariosDefault[$i])){
					
					//Checamos cuantas entregas estan dentro de los rangos de horarios a mosrar
					$numDeliveriesInHorario = 0;
					for($e=0;$e<count($deliveries);$e++){
						if($this->isTimeInRange($deliveries[$e]["deliverytimemiddle"], $horariosDefault[$i]["flo_horarios_timefrom"], $horariosDefault[$i]["flo_horarios_timeto"])){
							$numDeliveriesInHorario++;
						}
					}
					
					$cupo = ($horariosDefault[$i]["flo_horarios_deliverylimit"] - $numDeliveriesInHorario);
					//Checamos si esta en tiempo para entrega mismo dia
					
					if(!$this->isInTimeToDeliver($horariosDefault[$i]["flo_horarios_buytimebeforeshipment"], $dateToCheck." ".$horariosDefault[$i]["flo_horarios_timefrom"])){
						$cupo = 0;
					}else if($today == $dateToCheck){
						
						if(!$productsCanDeliverySameDay){
							$cupo = 0;
						}
					}else if($today > $dateToCheck){
						$cupo = 0;
					}
					if($cupo > 0){
						$text = "Disponible ";
						$text .= $dayText." ".$horariosDefault[$i]["flo_horarios_timefrom_int"]."".$horariosDefault[$i]["flo_horarios_timefrom_m"];
						$text .= " - ".$horariosDefault[$i]["flo_horarios_timeto_int"]."".$horariosDefault[$i]["flo_horarios_timeto_m"];
						return $text;
					}
					
					
				}
			}
			
			//Termino el bucle de horarios y no hubo cupo en ninguno
			return $this->getProductDisponibilityById($productId, date("Y-m-d", strtotime($dateToCheck." + 1 day")));
			
		}
		
	}
	
	
	/*
	* 
	* Get delivery price based on delivery date
	* 
	* Params $dateToCheck string with date to validate
	* Params $arrayPricesList array with all prices
	* Params $default bolean, define if horario is default or restricciones
	* Return $deliveryPrice float
	*/
	function getDeliveryPrice($dateToCheck="", $arrayPricesList=array(), $default=true){
		$today = date("Y-m-d");
		$tomorrow = date("Y-m-d", strtotime($today." + 1 day"));
		
		if($default==true){
			//return $arrayPricesList["flo_horarios_price"];//Mientras ajustan en produccion los precios
			if($dateToCheck == "" || count($arrayPricesList) == 0)	return $arrayPricesList["flo_horarios_price"];
			
			if($dateToCheck == $today){
				//Return prices TODAY
				return $arrayPricesList["flo_horarios_pricetoday"];
				
			}else if($dateToCheck == $tomorrow){
				//Return prices TOMORROW
				return $arrayPricesList["flo_horarios_pricetomorrow"];
				
			}else if($dateToCheck > $tomorrow){
				//Return normal prices
				return $arrayPricesList["flo_horarios_price"];
				
			}else if($dateToCheck < $today){
				//Return normal prices
				return $arrayPricesList["flo_horarios_price"];
				
			}else{
				//Return normal prices
				return $arrayPricesList["flo_horarios_price"];
			}
		}else{
			//return $arrayPricesList["flo_horario_restricciones_price"];//Mientras ajustan en produccion los precios
			if($dateToCheck == "" || count($arrayPricesList) == 0)	return $arrayPricesList["flo_horario_restricciones_price"];
			
			if($dateToCheck == $today){
				//Return prices TODAY
				return $arrayPricesList["flo_horario_restricciones_pricetoday"];
				
			}else if($dateToCheck == $tomorrow){
				//Return prices TOMORROW
				return $arrayPricesList["flo_horario_restricciones_pricetomorrow"];
				
			}else if($dateToCheck > $tomorrow){
				//Return normal prices
				return $arrayPricesList["flo_horario_restricciones_price"];
				
			}else if($dateToCheck < $today){
				//Return normal prices
				return $arrayPricesList["flo_horario_restricciones_price"];
				
			}else{
				//Return normal prices
				return $arrayPricesList["flo_horario_restricciones_price"];
			}
		}
		
		
	}
	
}//End Class